/*
    M2B_battleship_mex.cpp

    Copyright 2009 Dhruv Batra (batradhruv -at- cmu.edu, www.ece.cmu.edu/~dbatra)
    
    This file is the matlab wrapper for M2B_battleship.h/cpp (later combined into 
    M2B_battleship.hpp because matlab compiler wouldn't accept broken template definitions).

*/


#include "mex.h"
#include "matrix.h"
#include "M2B_battleship.hpp"

// Let Matlab handle memory allocation
#define malloc mxMalloc
#define calloc mxCalloc 
#define free mxFree
#define realloc mxRealloc
#define printf mexPrintf
#define assert mxAssert
#define error mexErrMsgTxt



/* ************************************************************************** 
   mex Wrapper
   ************************************************************************** */

/* Function: [o2b_node_en, o2b_edge_en, o2b_edge_list, offset] = M2B_battleship(o2m_node_en, o2m_edge_en, o2m_edge_list)

   ----------------------
   Inputs:
   o2m_node_en      nm_labels x nm_nodes matrix holding multilabel node/unary energy. Double. 
   o2m_edge_en      (nm_labels^2) x nm_edges matrix holding multilabel edge/pairwise energy. Double. 
   o2m_edge_list    2 x nm_edges matrix holding adjacency list for the multilabel graph. Uint32. (0-based indexing)
		  
   Outputs:
   o2b_node_en      2 x nb_nodes matrix holding boolean node/unary energy. Double. 
   o2b_edge_en      4 x nb_edges matrix holding boolean edge/pairwise energy. Double. 
   o2b_edge_list    2 x nb_edges matrix holding boolean graph adjacency list. Uint32. (0-based indexing)
   offset           scalar holding offset between boolean en and multilabel en. multi_en = bool_en + offset.

   ----------------------
*/

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  uint32_t nm_nodes, nm_edges, nm_labels, nm_labels_sqr;
  uint32_t nb_nodes, nb_edges;
  int row,col,i,j;
  mxArray *xdata;
  double *xvalues,*pr;
  mwSize dims[2];
  
  // Check inputs
  if (nrhs != 3) 
    error("Incorrect number of input arguments!: Should be 3");
  
  if ( (nlhs<3) || (nlhs>4) ) 
    error("Incorrect number of output arguments!: Should be 3 or 4");
  
  // Check pointers
  for (i=1; i<3; i++)
    assert(prhs[i], "Null pointers");

  if ( (!mxIsDouble(prhs[0])) || (!mxIsDouble(prhs[1])) || (!mxIsUint32(prhs[2])) )
    error("Incorrect datatype of input arguments");
      
  /* READ 1st INPUT -> 'o2m_node_en' */
  nm_labels = (uint32_t) mxGetM(prhs[0]);
  nm_nodes = (uint32_t) mxGetN(prhs[0]);
  
  /* READ 2nd INPUT -> 'o2m_edge_list' */
  nm_labels_sqr = (uint32_t) mxGetM(prhs[1]);
  nm_edges = (uint32_t) mxGetN(prhs[1]);

  if ( (nm_labels*nm_labels != nm_labels_sqr) || (mxGetM(prhs[2]) != 2) || (mxGetN(prhs[2]) != nm_edges) )
    error("Incorrect size of inputs");
  
  // Create space for output
  nb_nodes = nm_nodes*(nm_labels-1);
  nb_edges = nm_nodes*(nm_labels-2)+(nm_labels-1)*(nm_labels-1)*nm_edges;
  
  plhs[0] = mxCreateDoubleMatrix(2, nb_nodes, mxREAL);
  plhs[1] = mxCreateDoubleMatrix(4, nb_edges, mxREAL);
  
  //dims[0] = 2; dims[1] = nb_edges;
  plhs[2] = mxCreateNumericMatrix(2, nb_edges, mxUINT32_CLASS, mxREAL);
  
  assert( plhs[0] || plhs[1] || plhs[2] , "Null pointers");
  
  // Set up order2_multi object
  class order2_multi<double> multi_en(nm_nodes, nm_edges, nm_labels, \
				     mxGetPr(prhs[0]), mxGetPr(prhs[1]), (uint32_t*) mxGetPr(prhs[2]), \
				     mxGetPr(plhs[0]), mxGetPr(plhs[1]), (uint32_t*) mxGetPr(plhs[2]));
  
  // Transform
  multi_en.multi2bool();
  
  if (nlhs > 3)
    plhs[3] = mxCreateDoubleScalar(multi_en.offset);
}
